//TODO observe unload
//unload

A4J.AJAX._eventQueues = {};

var extend = function(target, source) {
	for (var property in source) {
		target[property] = source[property];
	}
};

//queue constructor
A4J.AJAX.EventQueue = function() {
	var Pipeline = function(size, gate, queue) {
		this.size = size || -1;
		
		this.gate = gate;
		this.queue = queue;
		
		this.items = new Array();
	};

	extend(Pipeline.prototype, function() {
		
		return {
			submit: function() {
				var data = this.items.shift();
				if (data) {
					this.createRequest(data);
				}
			},
			
			createRequest: function(data) {
				LOG.debug("Queue '" + this.queue.name + "' will submit request NOW");
				this.request = data.submit();
				this.similarityGroupingId = data.getSimilarityGroupingId();
				this.request.shouldNotifyQueue = true;
			},
			
			clearRequest: function() {
				this.request = undefined;
				this.similarityGroupingId = undefined;
			},
			
			getSize: function() {
				var size = this.items.length;
				
				if (this.request) {
					size++;
				}
				
				return size;
			},
			
			dropFirst: function() {
				this.items.shift();
			},
			
			fireFirst: function() {
				var data = this.items.shift();
				if (data) {
					data.submit()
				}
			},

			addEvent: function(data) {
				if (!this.isFull()) {
					this.items.push(data);
				} else {
					//log error
				}
			},
			
			//TODO new good name!
			submitEvent: function() {
				if (!this.request && !this.isEmpty()) {
					this.submit();
				}
			},
			
			submitNext: function() {
				this.clearRequest();
				LOG.debug("After request: queue '" + this.queue.name + "'");
				
				if (this.getSize() > 0) {
					this.submit();
				} else {
					this.gate.transferIfReady();
				}
			},
			
			isFull: function() {
				return this.getSize() == this.size;
			},
			
			isEmpty: function() {
				return this.getSize() == 0;
			},
			
			hasNext: function() {
				return this.items.length != 0;
			},
			
			abortCurrentRequest: function() {
				if (this.request) {
					LOG.debug("Current request in queue '" + this.queue.name + "' has been aborted");

					this.request.shouldNotifyQueue = false;
					this.request.abort();
					this.clearRequest();
				}
			},
			
			abortDupResponses: function(data) {
				if (!this.hasNext() && data.getSimilarityGroupingId() == this.similarityGroupingId) {
					this.abortCurrentRequest();
				}
			}
		}
	}());

	var Gate = function(queue) {
		this.queue = queue;
		
		this.eventsCounter = 0;
	};

	extend(Gate.prototype, {
		
		stopRequestDelay: function () {
			if (this.timer) {
				clearTimeout(this.timer);
				this.timer = undefined;
			}
		},
		
		resetRequestDelay: function() {
			this.delayPassed = false;
			
			this.stopRequestDelay();
			this.startRequestDelay();
		},
		
		startRequestDelay: function() {
			var delay = this.data.getRequestDelay();

			LOG.debug("Queue will wait " + (delay || 0) + "ms before submit");
			
			if (delay) {
				var _this = this;
				this.timer = setTimeout(function() {
					try {
						_this.transferIfEmpty();
						_this.timer = undefined;
					} finally {
						_this = undefined;
					}
				}, delay);
			} else {
				this.transferIfEmpty();
			}
		},
		
		transferIfEmpty: function() {
			this.delayPassed = true;

			if (this.pipeline.isEmpty()) {
				this.pipeline.addEvent(this.pop());
				this.pipeline.submitEvent();
			}
		},
		
		transferIfReady: function() {
			if (this.delayPassed) {
				this.pipeline.addEvent(this.pop());
				this.pipeline.submitEvent();
			}
		},
		
		pop: function() {
			var data = this.data;
			this.data = undefined;
			
			this.eventsCounter = 0;
			this.similarityGroupingId = undefined;
			this.delayPassed = false;
			
			return data;
		},

		push: function() {
			var DROP_NEW = 'dropNew';
			var DROP_NEXT = 'dropNext';
			var FIRE_NEW = 'fireNew';
			var FIRE_NEXT = 'fireNext';

			return function(data) {
				if (data.isIgnoreDupResponses()) {
					this.pipeline.abortDupResponses(data);
				}
				
				var similarityGroupingId = data.getSimilarityGroupingId();
				
				if (this.similarityGroupingId == similarityGroupingId) {
					LOG.debug("Similar request already in queue '" + this.queue.name + "'");
					this.data = data;
					data.setEventsCounter(++this.eventsCounter);
			
					this.resetRequestDelay();
				} else {
					LOG.debug("New event added to queue '" + this.queue.name + "'. Queue similarityGroupingId changed to " + similarityGroupingId);
					if (this.data) {
						this.stopRequestDelay();
						this.pipeline.addEvent(this.pop());
					}

					var newDataHandled = false;
					
					if (this.pipeline.isFull()) {
						var behavior = data.getSizeExceededBehavior();
						
						var queue = this.queue;
						var queueOptions = queue.queueOptions;
						
						if (queueOptions.onsizeexceeded) {
							var query = data.query;
							var options = data.options;
							var event = data.event;

							queueOptions.onsizeexceeded.call(this.queue, query, options, event);
						}
						
						if (behavior == DROP_NEW || 
								(behavior == DROP_NEXT && !(this.pipeline.hasNext()))) {
							
							newDataHandled = true;
						} else if (behavior == FIRE_NEW || 
								(behavior == FIRE_NEXT && !(this.pipeline.hasNext()))) {

							data.submit();
							newDataHandled = true;
						} else if (behavior == DROP_NEXT) {
							this.pipeline.dropFirst();
						} else if (behavior == FIRE_NEXT) {
							this.pipeline.fireFirst();
						}
					
					}
					
					this.pipeline.submitEvent();

					if (!newDataHandled) {
						this.data = data;
						this.similarityGroupingId = similarityGroupingId;
						this.startRequestDelay();
					}
				}
			}
		}()
	});

	return function(name, queueOptions, requestOptions) {
		this.name = name;
		this.queueOptions = queueOptions || {};
		this.requestOptions = requestOptions || {};
		
		this.gate = new Gate(this);
		this.pipeline = new Pipeline(this.queueOptions.size, this.gate, this);
		this.gate.pipeline = this.pipeline;
	};
}();

A4J.AJAX.EventQueue.DEFAULT_QUEUE_NAME = "org.richfaces.queue.global";

A4J.AJAX.EventQueue.getQueue = function(name) {
	return A4J.AJAX._eventQueues[name];
};

A4J.AJAX.EventQueue.addQueue = function(queue) {
	var name = queue.name;
	
	if (A4J.AJAX._eventQueues[name]) {
		throw "Queue already registered";
	} else {
		LOG.debug("Adding queue '" + name + "' to queues registry");
		A4J.AJAX._eventQueues[name] = queue;
	}
};

A4J.AJAX.EventQueue.removeQueue = function(name) {
	delete A4J.AJAX._eventQueues[name];
};

A4J.AJAX.EventQueue.getOrCreateQueue = function(){
	var qualifyName = function(name, prefix) {
		if (prefix) {
			return prefix + ':' + name;
		} else {
			return name;
		}
	};
	
	return function(options, formId) {
		var queueName = options.eventsQueue;
		var namespace = options.namespace;

		var formQueueName;
		var viewQueueName;
		var implicitQueueName;
		
		if (queueName) {
			formQueueName = qualifyName(queueName, formId);
			viewQueueName = qualifyName(queueName, namespace);
			
			implicitQueueName = viewQueueName;
		} else {
			formQueueName = formId;
			viewQueueName = qualifyName(A4J.AJAX.EventQueue.DEFAULT_QUEUE_NAME, namespace);
		
			implicitQueueName = options.implicitEventsQueue;
		}
		
		var queue = A4J.AJAX._eventQueues[formQueueName];
		if (!queue) {
			queue = A4J.AJAX._eventQueues[viewQueueName];
			if (!queue && implicitQueueName) {
				queue = A4J.AJAX._eventQueues[implicitQueueName];
				if (!queue) {
					LOG.debug("Creating new transient queue '" + implicitQueueName + "' with default settings");
					queue = new A4J.AJAX.EventQueue(implicitQueueName);
					queue._transient = true;
					
					A4J.AJAX.EventQueue.addQueue(queue);
				}
			}
		}
		
		return queue;
	}
}();

A4J.AJAX.EventQueue.prototype = function() {
	var EventQueueData = function(queue, query, options, event) {
		this.queue = queue;
		this.query = query;
		this.event = event;
		
		this.options = options || {};

		this.eventsCount = 1;
	};
	
	extend(EventQueueData.prototype, {
		submit: function() {
			this.query.appendParameter("AJAX:EVENTS_COUNT", this.eventsCount);
			this.request = A4J.AJAX.SubmitQuery(this.query, this.options)
			
			var queue = this.queue;
			this.request.queue = queue;

			if (this.options.queueonsubmit) {
				this.options.queueonsubmit.call(queue, this.request);
			}
			
			return this.request;
		},
	
		getSimilarityGroupingId: function() {
			return this.options.similarityGroupingId;
		},
		
		getRequestDelay: function() {
			return this.options.requestDelay;
		},
		
		getSizeExceededBehavior: function() {
			return this.queue.getSizeExceededBehavior();
		},
		
		isIgnoreDupResponses: function() {
			return this.options.ignoreDupResponses;
		},
		
		setEventsCounter: function(count) {
			this.eventsCount = count;
		}
	});
	

//	LOG.debug("Queue will wait " + (delay || 0) + "ms before submit");
//	LOG.debug("Similar request already in queue '" + this.name + "'");

//	LOG.debug("After request: queue '" + this.name + "' is empty now");
//	LOG.debug("Deleting transient queue '" + this.name + "' from queues registry");
//	LOG.debug("After request: queue not empty, processing next event in queue '" + this.name + "'");

//	LOG.debug("Queue '" + this.name + "' will submit request NOW");
//	LOG.debug("Delay for request not passed yet, have to wait");
//	LOG.debug("Queue '" + this.name + "' is empty, nothing to submit");
//	LOG.debug("Request in queue '" + this.name + "' is active now, wait until its completion");
//	LOG.debug("New event added to queue '" + this.name + "'. Queue similarityGroupingId changed to " + similarityGroupingId);
	
	var extendOptions = function(options) {
		var opts = {};
		
		for (var name in options) {
			opts[name] = options[name];
		}
		
		for (var name in this.requestOptions) {
			if (typeof opts[name] == 'undefined') {
				opts[name] = this.requestOptions[name];
			}
 		}
		
		return opts;
	};
	
	return {
		//TODO separate service functions
		
		getQueueSize: function() {
			return this.queueOptions.size;
		},
		
		getSizeExceededBehavior: function() {
			var policy = this.queueOptions.sizeExceededBehavior;
			if (!policy) {
				policy = 'dropNext';
			}

			return policy;
		},
		
		push: function(query, opts, event) {
			var options = extendOptions.call(this, opts);
			var queueData = new EventQueueData(this, query, options, event);
			
			this.gate.push(queueData);
		},

		pop: function() {
			this.pipeline.submitNext()
		}
	}
}();
