/**
 * License Agreement.
 *
 * Rich Faces - Natural Ajax for Java Server Faces (JSF)
 *
 * Copyright (C) 2007 Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.realworld.fileupload;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Synchronized;
import org.jboss.seam.faces.FacesMessages;
import org.richfaces.event.UploadEvent;
import org.richfaces.model.UploadItem;
import org.richfaces.realworld.domain.User;

/**
 * @author $Autor$
 *
 */
@Name("fileUploadBean")
@Scope(ScopeType.SESSION)
@Synchronized(timeout=2000)
public class FileUploadBean {
	
	@In(create=true)
	private FileManager fileManager;
	
	@In FacesMessages facesMessages;
	
	private ArrayList<FileItem> files = new ArrayList<FileItem>();
	private boolean autoUpload = false;
	public int getSize() {
		if (getFiles().size()>0){
			return getFiles().size();
		}else 
		{
			return 0;
		}
	}

	public synchronized void paint(OutputStream stream, Object object) throws IOException {
		stream.write(getFiles().get((Integer)object).getData());
	}
	
	public synchronized void listener(UploadEvent event) throws Exception{
	    UploadItem item = event.getUploadItem();
	    FileItem file = new FileItem();
	    file.setLength(item.getData().length);
	    file.setName(item.getFileName());
	    file.setData(item.getData());
	    file.setSelected(false);
	    files.add(file);
	}
	
	public void clearUploadData() {
		files.clear();
	}
	
	public void clearSelectedData() {
		for(int i = 0; i < files.size(); i++){
			if(files.get(i).isSelected()){
				files.remove(i);
			}
		}
	}
	
	public void storeSelectedData() {
		for(FileItem file:files){
			if(file.isSelected()){
				//TODO
				//Save file to disk
				//Update tree
				//Update domain model
			}
		}
	}
	
	public void storeAll() {
		for(FileItem file:files){
			//TODO
				//Save file to disk
				//Update tree
				//Update domain model
		}
	}
	
	public long getTimeStamp(){
		return System.currentTimeMillis();
	}
	
	public ArrayList<FileItem> getFiles() {
		return files;
	}

	public void setFiles(ArrayList<FileItem> files) { 
		this.files = files;
	}

	public boolean isAutoUpload() {
		return autoUpload;
	}

	public void setAutoUpload(boolean autoUpload) {
		this.autoUpload = autoUpload;
	}
}


