package org.richfaces.realworld.tree;

import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.imageio.ImageIO;

import org.ajax4jsf.context.AjaxContext;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Synchronized;
import org.richfaces.component.UIDragSupport;
import org.richfaces.component.UITree;
import org.richfaces.component.UITreeNode;
import org.richfaces.component.html.HtmlTree;
import org.richfaces.event.DataScrollerEvent;
import org.richfaces.event.DropEvent;
import org.richfaces.event.NodeSelectedEvent;
import org.richfaces.model.TreeNode;
import org.richfaces.model.TreeRowKey;
import org.richfaces.realworld.domain.User;
import org.richfaces.realworld.fileupload.FileManager;
import org.richfaces.realworld.util.NavigationEnum;

@Name("treeSelectionManager")
@Synchronized(timeout=2000)
@Scope(ScopeType.SESSION)
public class TreeSelectionManager {

	@In
	private User user;
	
	private NavigationEnum mainArea = NavigationEnum.IMAGE_PREVIEW;
	
	private TreeNode selectedImage;

	private TreeAlbumItem selectedAlbumItem;
	
	@In(create=true)
	FileManager fileManager;

	private String nodeTitle;

	public TreeAlbumItem getSelectedAlbumItem() {
		return selectedAlbumItem;
	}

	public void showFileUpload(TreeAlbumItem item){
		this.setSelectedAlbumItem(item);
		if(item.isLeaf()){
			this.setSelectedImage(null);
		}else{
			this.setSelectedImage(item.getImages().get(0));
		}
		this.setMainArea(NavigationEnum.FILE_UPLOAD);
	}
	
	public void scrollerListener(DataScrollerEvent event) {
		List<TreeNode> images = this.getSelectedAlbumItem().getImages();
		if (event.getNewScrolVal().equals("previous")) {
			for(int index = 0 ; index < images.size(); index++){
				if(images.get(index) == this.getSelectedImage()){
					this.setSelectedImage(images.get(index - 1));
				}
			}
		} else if (event.getNewScrolVal().equals("last")) {
			this.setSelectedImage(images.get(images.size() - 1));
		} else if (event.getNewScrolVal().equals("next")) {
			for(int index = 0 ; index < images.size(); index++){
				if(images.get(index) == this.getSelectedImage()){
					if(images.size() > index + 1){
						this.setSelectedImage(images.get(index + 1));
					}
				}
			}
		} else if (event.getNewScrolVal().equals("first")) {
			this.setSelectedImage(images.get(0));
		}

	}
	
	public synchronized void paint(OutputStream out, Object data) throws IOException{
		if (null == data) {
			return;
		}
		File imageResource = fileManager.getImage(user.getLogin() + fileManager.getFileSeparator() + this.getSelectedAlbumItem() + fileManager.getFileSeparator() + data.toString());
		if (imageResource != null) {
			InputStream paintData = new BufferedInputStream(new FileInputStream(imageResource));
			if (null == paintData) {
				// throw new Exception("");
				// TODO
			}
			BufferedImage image = ImageIO.read(paintData);
			ImageIO.write(image, "jpeg", out);
		}
	}
	
	public void dropListener(DropEvent dropEvent) {
		
		// resolve drag destination attributes
		UITreeNode destNode = (dropEvent.getSource() instanceof UITreeNode) ? (UITreeNode) dropEvent.getSource() : null;
		UITree destTree = destNode != null ? destNode.getUITree() : null;
		TreeRowKey dropNodeKey = (dropEvent.getDropValue() instanceof TreeRowKey) ? (TreeRowKey) dropEvent.getDropValue() : null;
		TreeNode droppedInNode = dropNodeKey != null ? destTree.getTreeNode(dropNodeKey) : null;

		// resolve drag source attributes
		UITreeNode srcNode = (dropEvent.getDraggableSource() instanceof UITreeNode) ? (UITreeNode) dropEvent.getDraggableSource() : null;
		UITree srcTree = srcNode != null ? srcNode.getUITree() : null;
		TreeRowKey dragNodeKey = (dropEvent.getDragValue() instanceof TreeRowKey) ? (TreeRowKey) dropEvent.getDragValue() : null;
		TreeNode draggedNode = dragNodeKey != null ? srcTree.getTreeNode(dragNodeKey) : null;
		if (dropEvent.getDraggableSource() instanceof UIDragSupport && srcTree == null && draggedNode == null && dropEvent.getDragValue() instanceof TreeNode) {	    
		    srcTree = destTree;
		    draggedNode = (TreeNode) dropEvent.getDragValue();
		    dragNodeKey = srcTree.getTreeNodeRowKey(draggedNode) instanceof TreeRowKey ? (TreeRowKey) srcTree.getTreeNodeRowKey(draggedNode) : null;
		}

		// Note: check if we dropped node on to itself or to item instead of
		// folder here
		if (droppedInNode != null && (droppedInNode.equals(draggedNode) || droppedInNode.getParent().getParent() != null || draggedNode.getParent().getParent() == null)) {
		    System.out.println("Warning: Can't drop on itself or to pic itself! Also can't move folders");
		    return;
		}

		if (dropNodeKey != null) {
		    // add destination node for rerender
		    destTree.addRequestKey(dropNodeKey);

		    Object state = null;
		    if (dragNodeKey != null) { // Drag from this or other tree
			TreeNode parentNode = draggedNode.getParent();
			// 1. remove node from tree
			state = srcTree.removeNode(dragNodeKey);
			// 2. add parent for rerender
			Object rowKey = srcTree.getTreeNodeRowKey(parentNode);
			srcTree.addRequestKey(rowKey);
			//TODO refactor this code
			if (dropEvent.getDraggableSource() instanceof UIDragSupport) {
			    // if node was gragged in it's parent place dragged node to
			    // the end of selected nodes in grid
			    if (droppedInNode.equals(parentNode)) {
			    }
			}
		    } else if (dropEvent.getDragValue() != null) { // Drag from some
								    // drag source
			draggedNode.setData(dropEvent.getDragValue().toString());
		    }

		    // generate new node id
		    Object id = getNewId(destTree.getTreeNode(dropNodeKey));
		    destTree.addNode(dropNodeKey, draggedNode, id, state);
		}

		AjaxContext ac = AjaxContext.getCurrentInstance();
		ac.addRenderedArea("mainform:imageList");
		// Add destination tree to reRender
		try {
		    ac.addComponentToAjaxRender(destTree);
		} catch (Exception e) {
		    System.err.print(e.getMessage());
		}
		
	    }
	
	private Object getNewId(TreeNode parentNode) {
		Map<Object, TreeNode> childs = new HashMap<Object, TreeNode>();
		Iterator<Map.Entry<Object, TreeNode>> iter = parentNode.getChildren();
		while (iter != null && iter.hasNext()) {
		    Map.Entry<Object, TreeNode> entry = iter.next();
		    childs.put(entry.getKey(), entry.getValue());
		}

		Integer index = 1;
		while (childs.containsKey(index)) {
		    index++;
		}
		return index;
	    }

	public void setSelectedAlbumItem(TreeAlbumItem selectedAlbumItem) {
		this.selectedAlbumItem = selectedAlbumItem;
	}

	public TreeNode getSelectedImage() {
		return selectedImage;
	}

	public void setSelectedImage(TreeNode selectedImage) {
		this.selectedImage = selectedImage;
	}

	public Boolean adviseNodeSelected(UITree tree) {
		TreeRowKey treeRowKey = (TreeRowKey) tree.getRowKey();
		TreeNode treeNode = (TreeNode) tree.getRowData(treeRowKey);
		
		if(treeNode instanceof TreeAlbumItem){
			TreeAlbumItem currentNode = (TreeAlbumItem)treeNode;
			if(this.getSelectedAlbumItem() != null){
				return treeNode == this.getSelectedAlbumItem() && null == this.getSelectedImage() ? Boolean.TRUE: Boolean.FALSE;
			}else{
				this.setSelectedAlbumItem(currentNode);
				if((currentNode).isLeaf()){
					return Boolean.TRUE ;
				}else{
					this.setSelectedImage((currentNode).getImages().get(0));
					return Boolean.FALSE ;
				}
			}
		}else if(treeNode instanceof TreeImageItem){
			return treeNode == this.getSelectedImage() ? Boolean.TRUE : Boolean.FALSE;
		}
		return null;
	}

	public Boolean adviseNodeOpened(UITree tree) {
		TreeRowKey treeRowKey = (TreeRowKey) tree.getRowKey();
		TreeNode treeNode = (TreeNode) tree.getRowData(treeRowKey);
		if(treeNode instanceof TreeAlbumItem){
			TreeAlbumItem currentNode = (TreeAlbumItem)treeNode;
			if(this.getSelectedAlbumItem() != null){
				return treeNode == this.getSelectedAlbumItem() ? Boolean.TRUE: Boolean.FALSE;
			}else{
				this.setSelectedAlbumItem(currentNode);
				if(!(currentNode).isLeaf()){
					this.setSelectedImage((currentNode).getImages().get(0));
				}
				return Boolean.TRUE ;
			}
		}
		return null;
	}

	public void processSelection(NodeSelectedEvent event) {
		HtmlTree tree = (HtmlTree) event.getComponent();
		if (tree.getRowData() instanceof TreeImageItem) {
			nodeTitle = ((TreeImageItem) (tree.getRowData())).getImage()
					.getName();
		} else if (tree.getRowData() instanceof TreeAlbumItem) {
			nodeTitle = ((TreeAlbumItem) (tree.getRowData())).getAlbum()
					.getName();
		}
		TreeNode<String> currentNode = tree.getModelTreeNode(tree.getRowKey());
		if (currentNode.isLeaf()) {
			selectedImage = currentNode;
			if(((TreeAlbumItem)currentNode.getParent()) != this.getSelectedAlbumItem()){
				this.setSelectedAlbumItem((TreeAlbumItem)currentNode.getParent());
			}
			return;
		} else {
			if(currentNode == this.getSelectedAlbumItem()){
				return;
			}else{
				setSelectedAlbumItem((TreeAlbumItem) currentNode);
				if(((TreeAlbumItem) currentNode).isLeaf()){
					this.setSelectedImage(null);
				}else{
					this.setSelectedImage(((TreeAlbumItem) currentNode).getImages().get(0));
				}
			}		
		}
	}

	public String getNodeTitle() {
		return nodeTitle;
	}

	public void setNodeTitle(String nodeTitle) {
		this.nodeTitle = nodeTitle;
	}

	public NavigationEnum getMainArea() {
		return mainArea;
	}

	public void setMainArea(NavigationEnum mainArea) {
		this.mainArea = mainArea;
	}
}
